@@ -1063,68 +1063,6 @@ void efi_cache_sync_image(unsigned long image_base,
 			  unsigned long alloc_size,
 			  unsigned long code_size);
 
-struct efi_smbios_record {
-	u8	type;
-	u8	length;
-	u16	handle;
-};
-
-const struct efi_smbios_record *efi_get_smbios_record(u8 type);
-
-struct efi_smbios_type1_record {
-	struct efi_smbios_record	header;
-
-	u8				manufacturer;
-	u8				product_name;
-	u8				version;
-	u8				serial_number;
-	efi_guid_t			uuid;
-	u8				wakeup_type;
-	u8				sku_number;
-	u8				family;
-};
-
-struct efi_smbios_type4_record {
-	struct efi_smbios_record	header;
-
-	u8				socket;
-	u8				processor_type;
-	u8				processor_family;
-	u8				processor_manufacturer;
-	u8				processor_id[8];
-	u8				processor_version;
-	u8				voltage;
-	u16				external_clock;
-	u16				max_speed;
-	u16				current_speed;
-	u8				status;
-	u8				processor_upgrade;
-	u16				l1_cache_handle;
-	u16				l2_cache_handle;
-	u16				l3_cache_handle;
-	u8				serial_number;
-	u8				asset_tag;
-	u8				part_number;
-	u8				core_count;
-	u8				enabled_core_count;
-	u8				thread_count;
-	u16				processor_characteristics;
-	u16				processor_family2;
-	u16				core_count2;
-	u16				enabled_core_count2;
-	u16				thread_count2;
-	u16				thread_enabled;
-};
-
-#define efi_get_smbios_string(__record, __type, __name) ({		\
-	int off = offsetof(struct efi_smbios_type ## __type ## _record,	\
-			   __name);					\
-	__efi_get_smbios_string((__record), __type, off);		\
-})
-
-const u8 *__efi_get_smbios_string(const struct efi_smbios_record *record,
-				  u8 type, int offset);
-
 void efi_remap_image(unsigned long image_base, unsigned alloc_size,
 		     unsigned long code_size);
 
@@ -1134,4 +1072,7 @@ void process_unaccepted_memory(u64 start, u64 end);
 void accept_memory(phys_addr_t start, phys_addr_t end);
 void arch_accept_memory(phys_addr_t start, phys_addr_t end);
 
+asmlinkage efi_status_t __efiapi
+efi_zboot_entry(efi_handle_t handle, efi_system_table_t *systab);
+
 #endif
