@@ -1540,11 +1540,6 @@ mptspi_probe(struct pci_dev *pdev, const struct pci_device_id *id)
 
 	scsi_scan_host(sh);
 
-#ifdef CONFIG_RHEL_DIFFERENCES
-	add_taint(TAINT_SUPPORT_REMOVED, LOCKDEP_STILL_OK);
-	pr_warn("MPTSPI MODULE IS NOT SUPPORTED\n");
-#endif
-
 	return 0;
 
 out_mptspi_probe:
